Example of use of startR
========================

Introduction
------------

This example shows an example of using `startR` including
- How to read data from a distributed data set
- How to read associated data of another data set matching the dates of the first one
- How to check the map results with `s2dverification::PlotEquiMap` and `s2dverification::PlotLayout`
- How to build a `startR` workflow
- How to run a `startR` workflow locally, locally on multicore, locally by chunks, locally by chunks and on multicore, on a remote cluster by chunks, and on a remote cluster and on multicore
- How to check the time series results with `s2dverification::PlotTimeSeries`

Code
----

```R
#install.packages('devtools')
#library(devtools)
#install_git('https://earth.bsc.es/gitlab/ces/multiApply', branch = 'master')
#install_git('https://earth.bsc.es/gitlab/es/easyNCDF', branch = 'master')
#install_git('https://earth.bsc.es/gitlab/es/startR', branch = 'develop-hotfixes-0.0.1-a')
```

```R
library(startR)

path <- '/esarchive/exp/ecearth/a0lg/monthly_mean/$var$_*/'
repos <- paste0(path, '$var$_*_S$sdate$_$member$_$chunk$.nc')
header <- Start(dat = repos,
                var = 'tas',
                member = indices(1:5),
                sdate = indices(1:21 * 2),
                chunk = 'all',
                ftime = indices(1:3),
                lat = 'all',
                lon = 'all',
                lat_reorder = Sort(),
                lon_reorder = CircularSort(0, 360),
                chunk_depends = 'sdate',
                ftime_across = 'chunk',
                merge_across_dims = TRUE,
                split_multiselected_dims = TRUE,
                synonims = list(ftime = 'time',
                                lat = c('lat', 'latitude'),
                                lon = c('lon', 'longitude')),
                return_vars = list(lon = NULL, 
                                   lat = NULL, 
                                   time = 'sdate')
               )

lon <- attr(header, 'Variables')$common$lon
lat <- attr(header, 'Variables')$common$lat
dates <- attr(header, 'Variables')$common$time

data <- eval(header)
#dim(data)
#   dat    var member  sdate  ftime    lat    lon 
#     1      1      5     21      3    256    512 
```

```R
#install_git('https://earth.bsc.es/gitlab/es/s2dverification', branch = 'master')
library(s2dverification)

# check 5 members
PlotLayout(PlotEquiMap, 
           c('lon', 'lat'),
           Subset(data, c('dat', 'var', 'sdate', 'ftime'), list(1, 1, 1, 1)),
           lon, 
           lat, 
           fill = FALSE
          )

# check 3 ftimes
PlotLayout(PlotEquiMap, 
           c('lon', 'lat'),
           Subset(data, c('dat', 'var', 'sdate', 'member'), list(1, 1, 1, 1)),
           lon, 
           lat, 
           fill = FALSE
          )

# check 21 sdates
PlotLayout(PlotStereoMap, 
           c('lon', 'lat'),
           Subset(data, c('dat', 'var', 'ftime', 'member'), list(1, 1, 1, 1)),
           lon, 
           lat, 
           fill = FALSE
          )
```

```R
path <- '/esarchive/recon/ecmwf/erainterim/monthly_mean/$var$_*/'
repos <- paste0(path, '$var$_$date$.nc')
data_obs <- Start(dat = repos,
              var = 'tas',
              date = unique(format(dates, '%Y%m')),
              ftime = values(dates),
              lat = 'all',
              lon = 'all',
              lat_reorder = Sort(),
              lon_reorder = CircularSort(0, 360),
              ftime_across = 'date',
              merge_across_dims = TRUE,
              split_multiselected_dims = TRUE,
              synonims = list(ftime = 'time',
                              lat = c('lat', 'latitude'),
                              lon = c('lon', 'longitude')),
              return_vars = list(lon = NULL, 
                                 lat = NULL) 
                                 #time = 'sdate')
             )

# TODO: Lons and lats are not being reordered internally in the arrays

data_obs <- eval(data_obs)
#dim(data_obs)
#  dat   var sdate ftime   lat   lon 
#    1     1    21     3   256   512 
```

```R
#install_git('https://earth.bsc.es/gitlab/es/s2dverification', branch = 'master')
library(s2dverification)

# check 3 ftimes
PlotLayout(PlotEquiMap, 
           c('lon', 'lat'),
           Subset(data_obs, c('dat', 'var', 'sdate'), list(1, 1, 1)),
           attr(data_obs, 'Variables')$common$lon, 
           attr(data_obs, 'Variables')$common$lat, 
           fill = FALSE
          )

# check 2 sdates
PlotLayout(PlotEquiMap, 
           c('lon', 'lat'),
           Subset(data_obs, c('dat', 'var', 'sdate', 'ftime'), list(1, 1, 1:2, 1)),
           attr(data_obs, 'Variables')$common$lon, 
           attr(data_obs, 'Variables')$common$lat,  
           fill = FALSE
          )
```

```R
# Compute area average of exp data
# Step, AddStep, Compute
source('https://earth.bsc.es/gitlab/es/s2dverification/raw/develop-Magic_WP6/R/WeightedMean.R')
s <- Step(.WeightedMean, c('lon', 'lat'), NULL)
wf <- AddStep(header, s, lon = lon, lat = lat, mask = NULL, weight = NULL)

# Compute locally
r <- Compute(wf)
#   user  system elapsed 
# 28.762   5.033  46.926

# Compute locally multicore
r <- Compute(wf, ncores = 4)
#   user  system elapsed 
#  9.620   0.878  21.483 

# Compute locally by chunks
r <- Compute(wf, chunks = list(sdate = 4))
#   user  system elapsed 
# 28.583   2.358  49.182

# Compute locally by chunks with multicore
r <- Compute(wf, ncores = 4, chunks = list(sdate = 4))
#   user  system elapsed 
# 11.486   1.592  32.452

# Compute cluster
r <- Compute(wf, chunks = list(sdate = 4), 
             cluster = list(queue_host = 'bsceslogin01', max_jobs = 4), 
             shared_dir = '/esnas/scratch/nmanuben/prova_workshop')

# Compute cluster
r <- Compute(wf, ncores = 4,
             chunks = list(sdate = 4), 
             cluster = list(queue_host = 'bsceslogin01', max_jobs = 4), 
             shared_dir = '/esnas/scratch/nmanuben/prova_workshop')
#   user  system elapsed 
#  2.971   1.963  81.467 
```

```R
#install.packages('ggplot2')
```

```R
library(ggplot2)
source('https://earth.bsc.es/gitlab/es/s2dverification/raw/develop-debug-plot-ts/R/PlotTimeSeries.R')

PlotTimeSeries(r[['output1']], coords = list(ftime = dates)) + scale_x_datetime()
```
